########################################################################
# Copyright (C) 2016 VMWare, Inc.                                      #
# All Rights Reserved                                                  #
########################################################################

import logging
import os
import shutil

log = logging.getLogger('Ramdisk')

from vmware.runcommand import runcommand, RunCommandError
from vmware.esximage import Errors
from .Misc import byteToStr

RAMDISK_ADD_CMD = 'localcli system visorfs ramdisk add -m 0 -M %s ' \
                  '-n %s -p 755 -t %s'
RAMDISK_RM_CMD = 'localcli system visorfs ramdisk list | grep /%s && ' \
                 'localcli system visorfs ramdisk remove -t %s'

def RemoveRamdisk(name, target):
   '''Unmount and remove a ramdisk.
   '''
   try:
      if os.path.exists(target):
         cmd = RAMDISK_RM_CMD % (name, target)
         rc, _ = runcommand(cmd)
         shutil.rmtree(target)
   except RunCommandError as e:
      log.warning('Failed to run %s: %s' % (cmd, e))
   except EnvironmentError as e:
      log.warning('Cannot remove %s directory: %s' % (target, e))

def CreateRamdisk(size, name, target):
   """Create and mount a ramdisk.
   """
   try:
      os.makedirs(target)
      cmd = RAMDISK_ADD_CMD % (size, name, target)
      rc, out = runcommand(cmd)
   except EnvironmentError as e:
      RemoveRamdisk(name, target)
      msg = 'Failed to create ramdisk %s' % name
      raise Errors.InstallationError(None, msg)
   except RunCommandError as e:
      RemoveRamdisk(name, target)
      msg = ('Failed to run %s: %s' % (cmd, e))
      raise Errors.InstallationError(None, msg)
   if rc != 0:
      RemoveRamdisk(name, target)
      msg = ('Failed to run %s: %s' % (cmd, byteToStr(out)))
      raise Errors.InstallationError(None, msg)

